from openfisca_france.model.base import (
    Variable, Individu, MONTH,
    TypesCategorieSalarie, TypesSecteurActivite, TypesStatutOccupationLogement,
    min_, where, set_input_dispatch_by_period, set_input_divide_by_period
    )


class mobili_jeune(Variable):
    value_type = float
    label = "Montant de l'aide au logement mobili-jeune"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.actionlogement.fr/l-aide-mobili-jeune'

    def formula_2022_06(individu, period, parameters):
        loyer = individu.menage('loyer', period)
        charges_locatives = individu.menage('charges_locatives', period)
        aide_logement = individu.famille('aide_logement', period)

        reste_a_charge = loyer + charges_locatives - aide_logement

        secteur_agricole = individu('secteur_activite_employeur', period) == TypesSecteurActivite.agricole

        maximum = where(secteur_agricole, parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.montant.maximum.secteur_agricole, parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.montant.maximum.autres_secteurs)

        eligibilite = individu('mobili_jeune_eligibilite', period)

        return where(
            (reste_a_charge >= parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.montant.minimum) * eligibilite,
            min_(reste_a_charge, maximum),
            0,
            )

    def formula_2012_07(individu, period, parameters):
        loyer = individu.menage('loyer', period)
        charges_locatives = individu.menage('charges_locatives', period)
        aide_logement = individu.famille('aide_logement', period)

        reste_a_charge = loyer + charges_locatives - aide_logement

        eligibilite = individu('mobili_jeune_eligibilite', period)

        return where(
            reste_a_charge >= parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.montant.minimum,
            eligibilite * min_(reste_a_charge, parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.montant.maximum.autres_secteurs),
            0,
            )


class mobili_jeune_eligibilite_employeur(Variable):
    value_type = bool
    label = "Conditions à remplir par l'employeur d'un demandeur d'aide au logement mobili-jeune"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.actionlogement.fr/l-aide-mobili-jeune'

    def formula_2012_07(individu, period):
        secteur_prive_non_agricole = (
            individu('categorie_salarie', period) == TypesCategorieSalarie.prive_non_cadre
            ) * (
                individu('secteur_activite_employeur', period) == TypesSecteurActivite.non_agricole
            )
        contributeur_peec = individu('peec_employeur', period)

        return secteur_prive_non_agricole * contributeur_peec


class mobili_jeune_eligibilite(Variable):
    value_type = bool
    label = "Éligibilité à l'aide au logement mobili-jeune"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.actionlogement.fr/l-aide-mobili-jeune'
    documentation = '''
    Conditions non modélisées :
    Etre locataire d'un logement en proximité géographique avec le lieu de la formation ou de l'entreprise.
    Avoir déposé la demande 3 mois avant la date de démarrage du cycle de formation ou jusqu’à 6 mois après cette date.
    Avoir un reste à charge de loyer après déduction d'APL/ALS supérieur ou égal à 10€.
    '''

    def formula_2022_06(individu, period, parameters):
        condition_age = individu('age', period) < parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.age_maximum

        alternant = individu('alternant', period)  # sous contrat d'apprentissage ou de professionnalisation
        contributeur_peec = individu('peec_employeur', period)

        smic_mensuel_brut = individu('smic_proratise', period)
        condition_remuneration = individu('salaire_de_base', period) <= smic_mensuel_brut

        statut_occupation_logement = individu.menage('statut_occupation_logement', period)
        locataire = (
            (statut_occupation_logement == TypesStatutOccupationLogement.locataire_hlm)
            + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_vide)
            + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_meuble)
            + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_foyer)
            )

        return condition_age * alternant * contributeur_peec * condition_remuneration * locataire

    def formula_2012_07(individu, period, parameters):
        condition_age = individu('age', period) < parameters(period).prestations_sociales.aides_jeunes.mobilite.mobili_jeune.age_maximum

        alternant = individu('alternant', period)  # sous contrat d'apprentissage ou de professionnalisation
        mobili_jeune_eligibilite_employeur = individu('mobili_jeune_eligibilite_employeur', period)

        smic_mensuel_brut = individu('smic_proratise', period)
        condition_remuneration = individu('salaire_de_base', period) <= smic_mensuel_brut

        statut_occupation_logement = individu.menage('statut_occupation_logement', period)
        locataire = (
            (statut_occupation_logement == TypesStatutOccupationLogement.locataire_hlm)
            + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_vide)
            + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_meuble)
            + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_foyer)
            )

        return condition_age * alternant * mobili_jeune_eligibilite_employeur * condition_remuneration * locataire



from openfisca_core import reforms
from openfisca_core.periods import MONTH, YEAR
from openfisca_core.entities import ADD, DIVIDE

from .. import entities


def brut_to_target(target_name = None, period = None, individu = None, **input_array_by_name):
    simulation = individu.simulation.clone(debug = individu.simulation.debug, trace = individu.simulation.trace)
    new_individu = simulation.individu
    new_individu.get_holder(target_name).delete_arrays()
    for variable_name, array in input_array_by_name.items():
        new_individu.get_holder(variable_name).put_in_cache(array, period)
    return new_individu(target_name, options = [ADD])


def build_reform(tax_benefit_system):
    from scipy.optimize import fsolve

    Reform = reforms.make_reform(
        key = 'inversion_revenus',
        name = 'Inversion des revenus',
        reference_tax_benefit_system = tax_benefit_system,
        )

    class salaire_imposable_pour_inversion(Reform.Variable):
        value_type = float
        entity = entities.Individu
        label = 'Salaire imposable utilisé pour remonter au salaire brut'
        definition_period = YEAR

    class chomage_imposable_pour_inversion(Reform.Variable):
        value_type = float
        entity = entities.Individu
        label = 'Autres revenus imposables (chômage, préretraite), utilisé pour l’inversion'
        definition_period = YEAR

    class retraite_imposable_pour_inversion(Reform.Variable):
        value_type = float
        entity = entities.Individu
        label = 'Pensions, retraites, rentes connues imposables, utilisé pour l’inversion'
        definition_period = YEAR

    class salaire_de_base(Reform.Variable):
        value_type = float
        entity = entities.Individu
        label = 'Salaire brut ou traitement indiciaire brut'
        reference = 'http://www.trader-finance.fr/lexique-finance/definition-lettre-S/Salaire-brut.html'
        definition_period = MONTH

        def formula(individu, period, parameters):
            '''Calcule le salaire brut à partir du salaire imposable ou sinon du salaire net.

            Sauf pour les fonctionnaires où il renvoie le traitement indiciaire brut
            Note : le supplément familial de traitement est imposable.
            '''
            # Get value for year and divide below.
            salaire_imposable_pour_inversion = individu.get_holder('salaire_imposable_pour_inversion').get_array(period.this_year)
            if salaire_imposable_pour_inversion is None:
                salaire_net = individu.get_holder('salaire_net').get_array(period)
                if salaire_net is not None:
                    # Calcule le salaire brut à partir du salaire net par inversion numérique.
                    if (salaire_net == 0).all():
                        # Quick path to avoid fsolve when using default value of input variables.
                        return salaire_net

                    def solve_function(salaire_de_base):
                        return brut_to_target(
                            target_name = 'salaire_net',
                            period = period,
                            salaire_de_base = salaire_de_base,
                            individu = individu,
                            ) - salaire_net
                    return fsolve(solve_function, salaire_net)

                salaire_imposable_pour_inversion = individu('salaire_imposable_pour_inversion',
                    period, options = [DIVIDE])

            # Calcule le salaire brut à partir du salaire imposable par inversion numérique.
            if (salaire_imposable_pour_inversion == 0).all():
                # Quick path to avoid fsolve when using default value of input variables.
                return salaire_imposable_pour_inversion

            def solve_function(salaire_de_base):
                return brut_to_target(
                    target_name = 'salaire_imposable',
                    period = period,
                    salaire_de_base = salaire_de_base,
                    individu = individu,
                    ) - salaire_imposable_pour_inversion

            return fsolve(solve_function, salaire_imposable_pour_inversion)

    #       TODO: inclure un taux de prime et calculer les primes en même temps que salaire_de_base

    #        # Calcule le salaire brut à partir du salaire imposable.
    #        # Sauf pour les fonctionnaires où il renvoie le traitement indiciaire brut
    #        # Note : le supplément familial de traitement est imposable.
    #
    #        hsup = simulation.calculate('hsup', period)
    #        categorie_salarie = simulation.calculate('categorie_salarie', period)
    #        P = parameters(period)
    #
    #        plafond_securite_sociale = P.cotsoc.gen.plafond_securite_sociale
    #
    #        salarie = P.cotsoc.sal.scale_tax_scales(plafond_securite_sociale)
    #        csg = P.csg.scale_tax_scales(plafond_securite_sociale)
    #
    #        salarie['noncadre'].update(salarie['commun'])
    #        salarie['cadre'].update(salarie['commun'])
    #
    #        # log.info(
    #        #     "Le dictionnaire des barèmes des cotisations salariés des titulaires de l'Etat contient : \n %s",
    #        #     salarie['fonc']["etat"],
    #        #     )
    #
    #        # Salariés du privé
    #
    #        noncadre = combine_tax_scales(salarie['noncadre'])
    #        cadre = combine_tax_scales(salarie['cadre'])
    #
    #        # On ajoute la CSG deductible
    #        noncadre.add_tax_scale(csg['activite']['deductible'])
    #        cadre.add_tax_scale(csg['activite']['deductible'])
    #
    #        nca = noncadre.inverse()
    #        cad = cadre.inverse()
    #        brut_nca = nca.calc(salaire_imposable_pour_inversion)
    #        brut_cad = cad.calc(salaire_imposable_pour_inversion)
    #        salbrut = brut_nca * (categorie_salarie == TypesCategorieSalarie['prive_non_cadre'])
    #        salbrut += brut_cad * (categorie_salarie == TypesCategorieSalarie['prive_cadre'])
    #
    #        # public etat
    #        # TODO: modifier la contribution exceptionelle de solidarité
    #        # en fixant son seuil de non imposition dans le barème (à corriger dans param.xml
    #        # et en tenant compte des éléments de l'assiette
    #        salarie['fonc']['etat']['excep_solidarite'] = salarie['fonc']['commun']['solidarite']
    #
    #        public_etat = salarie['fonc']['etat']['pension']
    #        # public_colloc = combine_tax_scales(salarie['fonc']["colloc"])  TODO
    #
    #        # Pour a fonction publique la csg est calculée sur l'ensemble salbrut(=TIB) + primes
    #        # Imposable = TIB - csg( (1+taux_prime)*TIB ) - pension(TIB) + taux_prime*TIB
    #        bareme_csg_titulaire_etat = csg['act']['deduc'].multiply_rates(1 + TAUX_DE_PRIME, inplace = False,
    #            new_name = "csg deduc titutaire etat")
    #        public_etat.add_tax_scale(bareme_csg_titulaire_etat)
    #        bareme_prime = MarginalRateTaxScale(name = "taux de prime")
    #        bareme_prime.add_bracket(0, -TAUX_DE_PRIME)  # barème équivalent à taux_prime*TIB
    #        public_etat.add_tax_scale(bareme_prime)
    #
    #        etat = public_etat.inverse()
    #
    #        # TODO: complete this to deal with the fonctionnaire
    #        # supplement_familial_traitement = 0  # TODO: dépend de salbrut
    #        # indemnite_residence = 0  # TODO: fix bug
    #
    #        # print 'salaire_imposable_pour_inversion', salaire_imposable_pour_inversion
    #        brut_etat = etat.calc(salaire_imposable_pour_inversion)
    #        # print 'brut_etat', brut_etat
    #        # print 'impot', public_etat.calc(brut_etat)
    #        # print 'brut_etat', brut_etat
    #        salbrut_etat = (brut_etat)
    #        # TODO: fonctionnaire
    #        # print 'salbrut_etat', salbrut_etat
    #        salbrut += salbrut_etat * (categorie_salarie == TypesCategorieSalarie['public_titulaire_etat'])
    #
    #        #<NODE desc= "Supplément familial de traitement " shortname="Supp. fam." code= "supplement_familial_traitement"/>
    #        #<NODE desc= "Indemnité de résidence" shortname="Ind. rés." code= "indemenite_residence"/>
    #        return salbrut + hsup

    class chomage_brut(Reform.Variable):
        value_type = float
        entity = entities.Individu
        label = 'Allocations chômage brutes'
        reference = 'http://vosdroits.service-public.fr/particuliers/N549.xhtml'
        definition_period = MONTH

        def formula(individu, period, parameters):
            '''"Calcule les allocations chômage brutes à partir des allocations imposables ou sinon des allocations nettes.
            '''
            # Get value for year and divide below.
            chomage_imposable_pour_inversion = individu.get_holder(
                'chomage_imposable_pour_inversion').get_array(period.this_year)
            if chomage_imposable_pour_inversion is None:
                chomage_net = individu.get_holder('chomage_net').get_array(period)
                if chomage_net is not None:
                    # Calcule les allocations chomage brutes à partir des allocations nettes par inversion numérique.
                    if (chomage_net == 0).all():
                        # Quick path to avoid fsolve when using default value of input variables.
                        return chomage_net

                    def solve_function(chomage_brut):
                        return brut_to_target(
                            chomage_brut = chomage_brut,
                            target_name = 'chomage_net',
                            period = period,
                            individu = individu,
                            ) - chomage_net
                    return fsolve(solve_function, chomage_net)

                chomage_imposable_pour_inversion = individu(
                    'chomage_imposable_pour_inversion', period, options = [DIVIDE])

            # Calcule les allocations chômage brutes à partir des allocations imposables.
            # taux_csg_remplacement = simulation.calculate('taux_csg_remplacement', period)
            if (chomage_imposable_pour_inversion == 0).all():
                # Quick path to avoid fsolve when using default value of input variables.
                return chomage_imposable_pour_inversion

            def solve_function(chomage_brut):
                return brut_to_target(
                    chomage_brut = chomage_brut,
                    # taux_csg_remplacement = taux_csg_remplacement,
                    target_name = 'chomage_imposable',
                    period = period,
                    individu = individu,
                    ) - chomage_imposable_pour_inversion
            return fsolve(solve_function, chomage_imposable_pour_inversion)

    class retraite_brute(Reform.Variable):
        value_type = float
        entity = entities.Individu
        label = 'Pensions de retraite brutes'
        reference = 'http://vosdroits.service-public.fr/particuliers/N20166.xhtml'
        definition_period = MONTH

        def formula(individu, period, parameters):
            '''"Calcule les pensions de retraite brutes à partir des pensions imposables ou sinon des pensions nettes.
            '''
            # Get value for year and divide below.
            retraite_imposable_pour_inversion = individu.get_holder(
                'retraite_imposable_pour_inversion').get_array(period.this_year)
            if retraite_imposable_pour_inversion is None:
                retraite_nette = individu.get_holder('retraite_nette').get_array(period)
                if retraite_nette is not None:
                    # Calcule les pensions de retraite brutes à partir des pensions nettes par inversion numérique.
                    if (retraite_nette == 0).all():
                        # Quick path to avoid fsolve when using default value of input variables.
                        return retraite_nette

                    def solve_function(retraite_brute):
                        return brut_to_target(
                            target_name = 'retraite_nette',
                            period = period,
                            retraite_brute = retraite_brute,
                            individu = individu,
                            ) - retraite_nette
                    return fsolve(solve_function, retraite_nette)

                retraite_imposable_pour_inversion = individu(
                    'retraite_imposable_pour_inversion', period, options = [DIVIDE])

            # Calcule les pensions de retraite brutes à partir des pensions imposables.
            taux_csg_remplacement = individu('taux_csg_remplacement', period)
            if (retraite_imposable_pour_inversion == 0).all():
                # Quick path to avoid fsolve when using default value of input variables.
                return retraite_imposable_pour_inversion

            def solve_function(retraite_brute):
                return brut_to_target(
                    retraite_brute = retraite_brute,
                    taux_csg_remplacement = taux_csg_remplacement,
                    target_name = 'retraite_imposable',
                    period = period,
                    individu = individu,
                    ) - retraite_imposable_pour_inversion
            return fsolve(solve_function, retraite_imposable_pour_inversion)

    return Reform()



#! /usr/bin/env python
'''
Extract mapping from subcommunes (arrondissements and communes associées) depcom codes to
commune depcom code.
'''


import argparse
import csv
import json
import logging
import os
import sys


app_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(app_name)


def main():
    parser = argparse.ArgumentParser(description = __doc__)
    parser.add_argument('insee_communes_file', help = 'INSEE communes file')
    parser.add_argument('-v', '--verbose', action = 'store_true', default = False, help = 'increase output verbosity')
    args = parser.parse_args()
    logging.basicConfig(level = logging.DEBUG if args.verbose else logging.WARNING)

    commune_depcom_by_subcommune_depcom = {}
    with open(args.insee_communes_file) as insee_communes_file:
        csv_reader = csv.DictReader(insee_communes_file, delimiter = '\t')
        for row in csv_reader:
            # Do not decode row as we read only ASCII.
            if row['ACTUAL'] in ('2', '5'):
                subcommune_depcom = row['DEP'] + row['COM']
                commune_depcom = row['POLE']
                commune_depcom_by_subcommune_depcom[subcommune_depcom] = commune_depcom

    logger.debug(json.dumps(commune_depcom_by_subcommune_depcom, encoding = 'utf-8', ensure_ascii = False, indent = 2))
    return 0


if __name__ == '__main__':
    sys.exit(main())



import json


from numpy import logical_or as or_, fromiter

from openfisca_france.model.base import *
from openfisca_france.france_taxbenefitsystem import COUNTRY_DIR


class taux_versement_transport(Variable):
    value_type = float
    entity = Individu
    label = ''
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        depcom_entreprise = individu('depcom_entreprise', period)
        effectif_entreprise = individu('effectif_entreprise', period)
        categorie_salarie = individu('categorie_salarie', period)

        seuil_effectif = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.versement_transport.seuil_effectif

        preload_taux_versement_transport()
        public = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)
            + (categorie_salarie == TypesCategorieSalarie.non_pertinent)
            )
        taux_versement_transport = fromiter(
            (
                get_taux_versement_transport(code_commune, period)
                for code_commune in depcom_entreprise
                ),
            dtype = 'float',
            )
        # "L'entreprise emploie-t-elle plus de 9 ou 10 salariés dans le périmètre de l'Autorité organisatrice de transport
        # (AOT) suivante ou syndicat mixte de transport (SMT)"
        return taux_versement_transport * or_(effectif_entreprise >= seuil_effectif, public) / 100


class versement_transport(Variable):
    value_type = float
    entity = Individu
    label = 'Versement transport'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        taux_versement_transport = individu('taux_versement_transport', period)
        cotisation = - taux_versement_transport * assiette_cotisations_sociales
        return cotisation


# File loading and parsing -> global table_versement_transport

def preload_taux_versement_transport():
    if 'table_versement_transport' not in globals():
        global table_versement_transport
        with open(COUNTRY_DIR + '/assets/versement_transport/taux.json') as data_file:
            table_versement_transport = json.load(data_file)


def get_taux_versement_transport(code_commune, period):
    if not isinstance(code_commune, str):  # In Python 3, code_commune is a bytes
        code_commune = code_commune.decode('utf-8')
    instant = period.start
    taux_commune = table_versement_transport.get(code_commune, None)
    if taux_commune is None:
        return 0.0 + 0.0
    else:
        aot = taux_commune.get('aot', None)
        smt = taux_commune.get('smt', None)
        return select_temporal_taux_versement_transport(aot, instant) + select_temporal_taux_versement_transport(smt, instant)


def select_temporal_taux_versement_transport(rates, instant):

    if rates is None:
        return 0.0

    taux = rates.get('taux')
    for date in sorted(taux, reverse=True):
        if str(instant) >= date:
            return float(taux[date])
    return 0.0



import datetime
from typing import Callable, List, Optional

import pytest

from openfisca_core import periods
from openfisca_core.reforms import Reform
from openfisca_france.scenarios import init_single_entity


@pytest.fixture
def new_scenario(axes, parent1, parent2, enfants) -> Callable[..., object]:
    def _scenario(
            reform: Reform,
            count: int,
            _max: int,
            _min: int,
            name: str,
            year: int,
            people: int
            ) -> object:

        return init_single_entity(
            reform.new_scenario(),
            axes = axes(count, _max, _min, name),
            period = periods.period(year),
            parent1 = parent1(year),
            parent2 = parent2(year, people),
            enfants = enfants(year, people),
            )

    return _scenario


@pytest.fixture
def axes() -> Callable[..., List[List[dict]]]:
    def _axes(count: int, _max: int, _min: int, name: str) -> List[List[dict]]:
        return [[
            dict(
                count = count,
                max = _max,
                min = _min,
                name = name,
                ),
            ]]

    return _axes


@pytest.fixture
def parent1() -> Callable[..., dict]:
    def _parent1(year: int) -> dict:
        return dict(date_naissance = datetime.date(year - 40, 1, 1))

    return _parent1


@pytest.fixture
def parent2() -> Callable[..., Optional[dict]]:
    def _parent2(year: int, people: int) -> Optional[dict]:
        if people >= 2:
            return dict(date_naissance = datetime.date(year - 40, 1, 1))

    return _parent2


@pytest.fixture
def enfants() -> Callable[..., List[dict]]:
    def _enfants(year: int, people: int) -> List[dict]:
        return [
            dict(date_naissance = datetime.date(year - 9, 1, 1))
            for number in (3, 4)
            if people >= number
            ]

    return _enfants



from openfisca_core.reforms import Reform

try:
    from scipy.optimize import fsolve
except ImportError:
    fsolve = None

from .. import entities
from ..model.base import *


def calculate_net_from(salaire_de_base, individu, period):

    # We're not wanting to calculate salaire_de_base again, but instead manually set it as an input variable
    individu.get_holder('salaire_de_base').put_in_cache(salaire_de_base, period)

    # Work in isolation
    temp_simulation = individu.simulation.clone()
    temp_individu = temp_simulation.individu

    # Force recomputing of salaire_net
    temp_individu.get_holder('salaire_net_a_payer').delete_arrays()

    net = temp_individu('salaire_net_a_payer', period)[0]

    return net


class salaire_de_base(Variable):
    value_type = float
    entity = entities.Individu
    label = 'Salaire brut'
    definition_period = MONTH

    def formula(individu, period, parameters):
        # Calcule le salaire brut à partir du salaire net par inversion numérique.

        net = individu.get_holder('salaire_net_a_payer').get_array(period)

        if net is None:
            return individu.empty_array()

        def solve_func(net):
            def innerfunc(essai):
                return calculate_net_from(essai, individu, period) - net
            return innerfunc
        brut_calcule = fsolve(
            solve_func(net),
            net * 1.5,  # on entend souvent parler cette méthode...
            xtol = 1 / 10  # précision
            )

        return brut_calcule


class de_net_a_brut(Reform):
    name = 'Inversion du calcul brut -> net'

    def apply(self):
        self.update_variable(salaire_de_base)
